#include "stdafx.h"


//----------------------------------------------------------------------------
//
//  Function:   vInitialize
//
//  Synopsis:   Reset the members of the class
//
//  Arguments:  -
//
//  Return:     -
//
//----------------------------------------------------------------------------
void    CTrStatMsh::vInitialize()
{
    m_BBVisibility.vSetBB(CVect3D(0,0,0), CVect3D(0,0,0));
    m_BBCollision.vSetBB(CVect3D(0,0,0), CVect3D(0,0,0));

    m_wFlags = 0;
}


//----------------------------------------------------------------------------
//
//  Function:   vRelease
//
//  Synopsis:   Release memory / Reset the members of the class
//
//  Arguments:  [boReleaseParent] -- true to release the parent class
//
//  Return:     -
//
//----------------------------------------------------------------------------
void    CTrStatMsh::vRelease(bool boReleaseParent /*= true*/)
{
    vInitialize();

    if(boReleaseParent) CTrLstMsh::vRelease(true);
}


//----------------------------------------------------------------------------
//
//  Function:   vStealData
//
//  Synopsis:   Transfer data from an object to the current one: after the
//              operation, the source object has lost his data member !
//
//  Arguments:  [obj] -- object to get the data from
//
//  Return:     -
//
//----------------------------------------------------------------------------
void    CTrStatMsh::vStealData(CTrStatMsh& obj)
{
    CTrLstMsh::vStealData(obj);

    m_BBVisibility  = obj.m_BBVisibility;
    m_BBCollision   = obj.m_BBCollision;
    m_wFlags        = obj.m_wFlags;

    obj.vInitialize();
}


//----------------------------------------------------------------------------
//
//  Function:   Constructor of class CTrStatMsh
//
//  Synopsis:   Construct a CTrStatMsh from a TR static mesh
//
//  Arguments:  [lpStaticMesh] -- static mesh to convert
//
//  Return:     -
//
//----------------------------------------------------------------------------
CTrStatMsh::CTrStatMsh(TR2_STATICMESH* lpStaticMesh, CTRFile* lpTRFile) : CTrLstMsh(VERSION_TRSTATICMESH,TrObj_Type_StaticMesh,(enumTrVersion)lpTRFile->GetTRType())
{
    vInitialize();

    // Construct CTrStatMsh
    vConstruct(lpTRFile,lpStaticMesh);
}


//----------------------------------------------------------------------------
//
//  Function:   Constructor of class CTrStatMsh
//
//  Synopsis:   Construct a CTrStatMsh from a block in memory
//
//  Arguments:  [lpSMsh] -- memory block
//              [boDeleteData] -- if true, delete the lpMVB pointer at the end
//
//  Return:     -
//
//----------------------------------------------------------------------------
CTrStatMsh::CTrStatMsh(BYTE* lpSMsh,bool boDeleteData) : CTrLstMsh(lpSMsh)
{
    vInitialize();

    if(lpSMsh)
        {
        BYTE* lpbData = lpSMsh + CTrLstMsh::dwGetDataSize();

        vConstruct(lpbData);

        if(boDeleteData) delete [] lpSMsh;
        }
}


//----------------------------------------------------------------------------
//
//  Function:   dwGetDataSize
//
//  Synopsis:   Get the size of the CTrStatMsh when put in a continuous block
//              of memory
//
//  Arguments:  -
//
//  Return:     size
//
//----------------------------------------------------------------------------
DWORD   CTrStatMsh::dwGetDataSize()
{
    DWORD   dwSize =    CTrLstMsh::dwGetDataSize() +
                        (sizeof(float) * 3 * 2 ) * 2 +  /* visibility & collision bounding boxes */
                        sizeof(WORD);                   /* m_wFlags */

    return  dwSize;
}


//----------------------------------------------------------------------------
//
//  Function:   lpbGetData
//
//  Synopsis:   Convert a CTrStatMsh to a continuous block of memory
//
//  Arguments:  [dwDataSize] -- on return, hold the size of the block
//
//  Return:     pointer to the memory block
//
//  Caution:    you must make a delete [] on the pointer returned when you
//              have finished working with it !
//----------------------------------------------------------------------------
BYTE*   CTrStatMsh::lpbGetData(DWORD& dwDataSize)
{
    BYTE*   lpbData = CTrLstMsh::lpbGetData(dwDataSize);
    DWORD   dwTrLstMshDataSize = CTrLstMsh::dwGetDataSize();
    BYTE*   lpbDataSave = lpbData;

    lpbData += dwTrLstMshDataSize;

    // Bounding boxes (visibility & collision)
    for(DWORD dwJ = 0; dwJ < 3; dwJ++)
        {
        *(float*)lpbData    = (float)m_BBVisibility.GetBBLo()[dwJ]; lpbData += sizeof(float);
        }

    for(dwJ = 0; dwJ < 3; dwJ++)
        {
        *(float*)lpbData    = (float)m_BBVisibility.GetBBHi()[dwJ]; lpbData += sizeof(float);
        }

    for(dwJ = 0; dwJ < 3; dwJ++)
        {
        *(float*)lpbData    = (float)m_BBCollision.GetBBLo()[dwJ]; lpbData += sizeof(float);
        }

    for(dwJ = 0; dwJ < 3; dwJ++)
        {
        *(float*)lpbData    = (float)m_BBCollision.GetBBHi()[dwJ]; lpbData += sizeof(float);
        }


    // Flags
    *(WORD*)lpbData = m_wFlags;         lpbData += sizeof(WORD);

    return lpbDataSave;
}


//----------------------------------------------------------------------------
//
//  Function:   boLoad
//
//  Synopsis:   Load a CTrStatMsh from disk
//
//  Arguments:  [cstrFileName] -- name of the file on disk
//
//  Return:     the object loaded & initialized
//
//----------------------------------------------------------------------------
/*static*/ CTrStatMsh  CTrStatMsh::objLoad(CString& cstrFileName)
{
    BYTE*   lpbData = CTrObjBase::lpbLoad(cstrFileName);

    return CTrStatMsh(lpbData,true);
}


//----------------------------------------------------------------------------
//
//  Function:   vConstruct
//
//  Synopsis:   Construct a CTrStatMsh from a block in memory
//
//  Arguments:  [lpObjMvb] -- memory block
//
//  Return:     -
//
//----------------------------------------------------------------------------
void    CTrStatMsh::vConstruct(BYTE* lpObjMvb)
{
    BYTE* lpbData = lpObjMvb;

    // Bounding boxes (visibility & collision)
    float* lpf = (float*) lpbData;

    m_BBVisibility.vSetBB(CVect3D(lpf[0],lpf[1],lpf[2]),CVect3D(lpf[3],lpf[4],lpf[5]));
    m_BBCollision.vSetBB (CVect3D(lpf[6],lpf[7],lpf[8]),CVect3D(lpf[9],lpf[10],lpf[11]));

    lpbData += sizeof(float) * 6 * 2;

    // Flags
    m_wFlags = *(WORD*)lpbData;         lpbData += sizeof(WORD);
}


//----------------------------------------------------------------------------
//
//  Function:   vConstruct
//
//  Synopsis:   Construct a CTrStatMsh from a TR static mesh
//
//  Arguments:  [lpStaticMesh] -- static mesh to convert
//
//  Return:     -
//
//----------------------------------------------------------------------------
void    CTrStatMsh::vConstruct(CTRFile* lpTRFile, TR2_STATICMESH* lpStaticMesh)
{
    // Get the mesh
    TR_FILE&    trFile = lpTRFile->GetTRFile();
    TR123_MESH* LstMeshes[] = { trFile.tr123File.lplpMeshes[trFile.tr123File.lpMeshPointers[lpStaticMesh->wMesh]] };

    if(trFile.tr123File.lpMeshPointers[lpStaticMesh->wMesh] == 0) LstMeshes[0] = NULL;

    CTrLstMsh::vConstruct(lpTRFile,1,LstMeshes);

    // Static mesh data
    vSetObjectId(lpStaticMesh->dwObjectId);

    m_BBVisibility.vSetBBLo(CVect3D(lpStaticMesh->BoundingBox[0].sMinX,-lpStaticMesh->BoundingBox[0].sMaxY,-lpStaticMesh->BoundingBox[0].sMaxZ));
    m_BBVisibility.vSetBBHi(CVect3D(lpStaticMesh->BoundingBox[0].sMaxX,-lpStaticMesh->BoundingBox[0].sMinY,-lpStaticMesh->BoundingBox[0].sMinZ));

    m_BBCollision.vSetBBLo (CVect3D(lpStaticMesh->BoundingBox[1].sMinX,-lpStaticMesh->BoundingBox[1].sMaxY,-lpStaticMesh->BoundingBox[1].sMaxZ));
    m_BBCollision.vSetBBHi (CVect3D(lpStaticMesh->BoundingBox[1].sMaxX,-lpStaticMesh->BoundingBox[1].sMinY,-lpStaticMesh->BoundingBox[1].sMinZ));

    m_wFlags                = lpStaticMesh->wFlags;
}
